@@ -6,20 +6,22 @@
 #include <assert.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <errno.h>
 
 #define err(x) perror(x),exit(1)
 
 int main(void)
 {
 	int max = numa_max_node();
+	int maxmask = numa_num_possible_nodes();
 	struct bitmask *nodes, *mask;
 	int pagesize = getpagesize();
 	int i;
 	int pol;
 	int node;
 	int err = 0;
-	nodes = numa_bitmask_alloc(max+1);
-	mask = numa_bitmask_alloc(max+1);
+	nodes = numa_bitmask_alloc(maxmask);
+	mask = numa_bitmask_alloc(maxmask);
 
 	for (i = max; i >= 0; --i) { 
 		char *mem = mmap(NULL, pagesize*(max+1), PROT_READ|PROT_WRITE, 
@@ -33,7 +35,7 @@ int main(void)
 
 		numa_bitmask_clearall(nodes);
 		numa_bitmask_clearall(mask);
-		numa_bitmask_setbit(mask, i);
+		numa_bitmask_setbit(nodes, i);
 
 		if (mbind(adr,  pagesize, MPOL_PREFERRED, nodes->maskp,
 							nodes->size, 0) < 0)
